/**
 * This library was based on Arduino RFM70 library. 
 *
 * Previously Daniel Weber rewrote it to not use additional 
 * Arduino libraries.
 *
 * I have cleaned it up, rewrote a bit and add more functions
 * that should make it easier for user to run communication
 * over theirs RFM70 modules.
 *
 * Contact to Daniel Weber:
 * Blog: http://projects.web4clans.com
 * Email: daniel.weber@web4clans.com
 *
 * Contact to me:
 * Website: http://www.srcpro.pl
 * e-mail: piotr.krzyzaniak@gmail.com
 *
 * Modified in 2016
 */

#include "rfm70.h"

// If you don't have a header file containing global definitions
// then define necessary signals here
#ifndef _GLOBAL_H_
  #define DDR_MOTOR       DDRA
  #define DDR_SPI         DDRB
  #define DDR_CE          DDRB
  #define DDR_IRQ         DDRD
  #define PORT_SPI        PORTB
  #define PORT_CE         PORTB
  #define PORT_IRQ        PORTD
  
  #define CE              2
  #define CSN             3
  #define SCK             7
  #define MISO            6
  #define MOSI            5
  #define IRQ             2
#endif

// Register initialization values and command macros

//************ Address definition commands
const uint8_t PROGMEM RFM70_cmd_adrRX0[] = { (0x20|0x0A), 0x34,0x43,0x10,0x10,0x01};
const uint8_t PROGMEM RFM70_cmd_adrTX[]  = { (0x20|0x10), 0x34,0x43,0x10,0x10,0x01};
const uint8_t PROGMEM RFM70_cmd_adrRX1[] = { (0x20|0x0B), 0x35,0x43,0x10,0x10,0x02};

//************ Bank0 register initialization commands
 const uint8_t PROGMEM RFM70_bank0Init[][2] = {
  // address data
  { (0x20|0x00), 0x7E }, //Enable CRC ,CRC=2byte, POWER UP, TX
  { (0x20|0x01), 0x03 }, //Enable auto acknowledgement data pipe0-5
  { (0x20|0x02), 0x03 }, //Enable RX Addresses pipe0-5
  { (0x20|0x03), 0x03 }, //RX/TX address field width 5byte
  { (0x20|0x04), 0x08 }, //x = 250 ms, y = 8 tries
  { (0x20|0x05), 0x08 }, //channel
  { (0x20|0x06), 0x3F }, //air data rate-2M,out power 5dbm,setup LNA gain high (0dBM)
  { (0x20|0x07), 0x00 }, //
  { (0x20|0x08), 0x00 }, //
  { (0x20|0x09), 0x00 }, //
  { (0x20|0x0C), 0xc3 }, //LSB Addr pipe 2
  { (0x20|0x0D), 0xc4 }, //LSB Addr pipe 3
  { (0x20|0x0E), 0xc5 }, //LSB Addr pipe 4
  { (0x20|0x0F), 0xc6 }, //LSB Addr pipe 5
  { (0x20|0x11), 0x20 }, //Payload len pipe0
  { (0x20|0x12), 0x20 }, //Payload len pipe1
  { (0x20|0x13), 0x20 }, //Payload len pipe2
  { (0x20|0x14), 0x20 }, //Payload len pipe3
  { (0x20|0x15), 0x20 }, //Payload len pipe4
  { (0x20|0x16), 0x20 }, //Payload len pipe5
  { (0x20|0x17), 0x00 }, //
  { (0x20|0x1C), 0x3F }, //Enable dynamic payload legth data pipe0-5
  { (0x20|0x1D), 0x07 }  //Enables Dynamic Payload Length,Enables Payload with ACK
};

//************ Bank1 register initialization commands
 const uint8_t PROGMEM RFM70_bank1Init[][5] = {
  // address data
  { (0x20|0x00), 0x40, 0x4B, 0x01, 0xE2 },
  { (0x20|0x01), 0xC0, 0x4B, 0x00, 0x00 },
  { (0x20|0x02), 0xD0, 0xFC, 0x8C, 0x02 },
  { (0x20|0x03), 0x99, 0x00, 0x39, 0x41 },
  { (0x20|0x04), 0xD9, 0x9E, 0x86, 0x0B },
  { (0x20|0x05), 0x24, 0x06, 0x7F, 0xA6 },
  { (0x20|0x06), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x07), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x08), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x09), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x0a), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x0b), 0x00, 0x00, 0x00, 0x00 },
  { (0x20|0x0C), 0x00, 0x12, 0x73, 0x00 },
  { (0x20|0x0D), 0x36, 0xb4, 0x80, 0x00 }
};

//************ Bank1 register 14 initialization commands
 const uint8_t PROGMEM RFM70_bank1R0EInit[] = {
  // address Data...
  (0x20|0x0E), 0x41,0x20,0x08,0x04,0x81,0x20,0xCF,0xF7,0xFE,0xFF,0xFF
};


//************ other commands: { <command>, <data>, ... }
const uint8_t PROGMEM RFM70_cmd_switch_cfg[] = { 0x50, 0x53 };  // switch Register Bank
const uint8_t PROGMEM RFM70_cmd_flush_rx[] = { 0xe2, 0x00 };    // flush RX FIFO
const uint8_t PROGMEM RFM70_cmd_flush_tx[] = { 0xe1, 0x00 };    // flush TX FIFO
const uint8_t PROGMEM RFM70_cmd_activate[] = { 0x50, 0x73 };    // Activation command


void rfm70_defaultTxInit(void)
{
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_enableTxIrq();
  rfm70_setModeTX();
}


void rfm70_defaultRxInit(void)
{
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_enableRxIrq();
  rfm70_setModeRX();
}


void rfm70_txWithAckInit(void)
{
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_enableTxIrq();
  rfm70_setModeTX();
}


void rfm70_rxWithAckInit(void)
{
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_enableRxIrq();
  rfm70_setModeRX();
}


void rfm70_txSimpleTransmissionInit(void)
{
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_disablePipe(RFM70_ALL_PIPES);
  rfm70_disableAutoAck(RFM70_ALL_PIPES);
  rfm70_disableDynamicPayload(RFM70_ALL_PIPES);
  rfm70_disableFeatures();
  rfm70_enableTxIrq();
  rfm70_setModeTX();
}


void rfm70_rxSimpleTransmissionInit(uint8_t payloadLen)
{
  // In simple configuration all pipes will have the same payload length
  rfm70_init(RFM70_DEFAULT_SPI_CLOCK_DIV);
  rfm70_disablePipe(RFM70_ALL_PIPES);
  rfm70_disableAutoAck(RFM70_ALL_PIPES);
  rfm70_disableDynamicPayload(RFM70_ALL_PIPES);
  rfm70_disableFeatures();
  rfm70_enablePipe(RFM70_PIPE_0);
  rfm70_enablePipe(RFM70_PIPE_1);
  rfm70_enablePipe(RFM70_PIPE_2);
  rfm70_enablePipe(RFM70_PIPE_3);
  rfm70_enablePipe(RFM70_PIPE_4);
  rfm70_enablePipe(RFM70_PIPE_5);
  rfm70_enableRxIrq();
  rfm70_setPayloadLen(RFM70_PIPE_0, payloadLen);
  rfm70_setModeRX();
}


void rfm70_init(uint8_t clk_div)
{
  rfm70_initHardware();
  rfm70_initSPI(clk_div);
  delay_ms(RFM70_BEGIN_INIT_WAIT_MS);
  rfm70_initRegisters();
}


void rfm70_initHardware(void)
{
  // set the CE ddr to output
  DDR_CE |= (1<<CE);

  // IRQ will be input activated at falling slope
  DDR_IRQ &= ~(1<<IRQ);

  // CE = 0 - Standby
  // CE = 1 - PTX or PRX;
  // init in PowerDown or StandBy mode to make writing to registers possible
  PORT_CE &= ~(1<<CE);
}


void rfm70_initRegisters(void)
{
  uint8_t i;

  // init bank 0 registers
  rfm70_selectBank(0);

  // To be sure we are in StandBy or PowerDown mode
  PORT_CE &=~ (1<<CE);
  for (i = 0; i < 20; i++) {
    rfm70_writeRegVal(pgm_read_byte(&RFM70_bank0Init[i][0]), pgm_read_byte(&RFM70_bank0Init[i][1]));
  }

  // init address registers in bank 0
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_adrRX0, sizeof(RFM70_cmd_adrRX0));
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_adrRX1, sizeof(RFM70_cmd_adrRX1));
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_adrTX, sizeof(RFM70_cmd_adrTX));

  // activate Feature register might be done only in standby or power down mode
  if(!rfm70_readRegVal(RFM70_REG_FEATURE)) {
    rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_activate, sizeof(RFM70_cmd_activate));
  }

  // now set Registers 1D and 1C
  rfm70_writeRegVal(pgm_read_byte(&RFM70_bank0Init[22][0]), pgm_read_byte(&RFM70_bank0Init[22][1]));
  rfm70_writeRegVal(pgm_read_byte(&RFM70_bank0Init[21][0]), pgm_read_byte(&RFM70_bank0Init[21][1]));

  // init bank 1 registers
  rfm70_selectBank(1);

  for (i=0; i < 14; i++) {
    rfm70_writeRegPgmBuf((uint8_t *)RFM70_bank1Init[i], sizeof(RFM70_bank1Init[i]));
  }

  // set ramp curve
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_bank1R0EInit, sizeof(RFM70_bank1R0EInit));
  delay_ms(RFM70_END_INIT_WAIT_MS);

  rfm70_selectBank(0);
}


void rfm70_selectBank(uint8_t bank)
{
  uint8_t current_bank;

  current_bank = rfm70_readRegVal(RFM70_REG_STATUS) & RFM70_STATUS_RBANK;

  if(current_bank) {  // bank == 1
    if(!bank)  {  // choosen bank == 0
      rfm70_writeRegVal(RFM70_CMD_ACTIVATE, 0x53);  //Then toggle value
    }
  }

  else {        // bank == 0
    if(bank) {    // choosen bank == 1
      rfm70_writeRegVal(RFM70_CMD_ACTIVATE, 0x53);  //Then toggle value
    }
  }
}


void rfm70_setMode(uint8_t mode) 
{
  if (mode == 1)
    rfm70_setModeRX();
  else
    rfm70_setModeTX();
}


uint8_t rfm70_getMode(void) 
{
  return rfm70_readRegVal(RFM70_REG_CONFIG) & RFM70_CONFIG_PRIM_RX;
}


void rfm70_setModeRX(void)
{
  uint8_t val;

  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val |= RFM70_CONFIG_PRIM_RX;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);

  // Flush RX FIFO
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_flush_rx, sizeof(RFM70_cmd_flush_rx)); 
}


void rfm70_setModeTX(void)
{
  uint8_t val;

  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val &= ~RFM70_CONFIG_PRIM_RX;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);

  // Flush TX FIFO
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_flush_tx, sizeof(RFM70_cmd_flush_tx));
}


void rfm70_setChannel(uint8_t cnum)
{
  rfm70_writeRegVal( RFM70_CMD_WRITE_REG | RFM70_REG_RF_CH, cnum);
}


uint8_t rfm70_getChannel(void) 
{
  return rfm70_readRegVal(RFM70_REG_RF_CH);
}


void rfm70_enableAutoAck(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_EN_AA);
  val |= pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_EN_AA, val);
}


void rfm70_disableAutoAck(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_EN_AA);
  val &= ~pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_EN_AA, val);
}


void rfm70_enableDynamicPayload(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_DYNPD);
  val |= pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_DYNPD, val);
}


void rfm70_disableDynamicPayload(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_DYNPD);
  val &= ~pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_DYNPD, val);
}


void rfm70_enablePipe(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_EN_RXADDR);
  val |= pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_EN_RXADDR, val);
}


void rfm70_disablePipe(uint8_t pipeNumber)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_EN_RXADDR);
  val &= ~pipeNumber;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_EN_RXADDR, val);
}


void rfm70_enableTxIrq(void)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val &= ~(RFM70_CONFIG_MASK_TX_DS | RFM70_CONFIG_MASK_MAX_RT);
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);
}


void rfm70_disableTxIrq(void)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val |= (RFM70_CONFIG_MASK_TX_DS | RFM70_CONFIG_MASK_MAX_RT);
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);
}


void rfm70_enableRxIrq(void)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val &= ~RFM70_CONFIG_MASK_RX_DR;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);
}


void rfm70_disableRxIrq(void)
{
  uint8_t val;
  
  val = rfm70_readRegVal(RFM70_REG_CONFIG);
  val |= RFM70_CONFIG_MASK_RX_DR;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_CONFIG, val);
}


void rfm70_enableFeatures(void)
{
  uint8_t val;

  val = rfm70_readRegVal(RFM70_REG_FEATURE);
  val |= RFM70_FEATURE_EN_DPL | RFM70_FEATURE_EN_ACK_PAY | RFM70_FEATURE_EN_DYN_ACK;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_FEATURE, val);
}


void rfm70_disableFeatures(void)
{
  uint8_t val;

  val = rfm70_readRegVal(RFM70_REG_FEATURE);
  val &= ~(RFM70_FEATURE_EN_DPL | RFM70_FEATURE_EN_ACK_PAY | RFM70_FEATURE_EN_DYN_ACK);
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_FEATURE, val);
}


void rfm70_setPayloadLen(uint8_t pipeNumber, uint8_t payloadLen)
{
  if(pipeNumber == RFM70_PIPE_0) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P0, payloadLen);
  } else if (pipeNumber == RFM70_PIPE_1) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P1, payloadLen);
  } else if (pipeNumber == RFM70_PIPE_2) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P2, payloadLen);
  } else if (pipeNumber == RFM70_PIPE_3) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P3, payloadLen);
  } else if (pipeNumber == RFM70_PIPE_4) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P4, payloadLen);
  } else if (pipeNumber == RFM70_PIPE_5) {
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RX_PW_P5, payloadLen);
  } else {
    // Do nothing
  }
}


void rfm70_configRfPower(uint8_t pwr) 
{
  if (pwr > 3) return;
  uint8_t tmp = rfm70_readRegVal(RFM70_REG_RF_SETUP);
  tmp &= 0xF9;
  tmp |= pwr << 1;
  rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_RF_SETUP, tmp);
}


uint8_t rfm70_getRfPower(void)
{	
  return (rfm70_readRegVal(RFM70_REG_RF_SETUP)&0x06) >> 1;
}


void rfm70_flushTxFIFO(void) 
{
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_flush_tx, sizeof(RFM70_cmd_flush_tx)); // Flush TX FIFO
}


void rfm70_flushRxFIFO(void) 
{
  rfm70_writeRegPgmBuf((uint8_t *)RFM70_cmd_flush_rx, sizeof(RFM70_cmd_flush_rx)); // Flush RX FIFO
}


uint8_t rfm70_sendPayload(uint8_t *payload, uint8_t len, uint8_t toAck)
{
  rfm70_readRegVal(RFM70_REG_CONFIG);
  // check TX_FIFO
  if (rfm70_readRegVal(RFM70_REG_FIFO_STATUS) & RFM70_FIFO_STATUS_TX_FULL) {
    return 0;
  }

  // send payload
  PORT_SPI &=~ (1<<CSN);

  if(toAck == PAYLOAD_WITH_ACK) {
    // Used in RX mode. Write payload together with ACK to pipe
    rfm70_transmitSPI(RFM70_CMD_W_ACK_PAYLOAD);
  }
  else if (toAck == PAYLOAD_WITH_NO_ACK) {
    // Used in TX mode. Disables AUTOACK on this specific packet
    rfm70_transmitSPI(RFM70_CMD_W_TX_PAYLOAD_NOACK);
  }
  else {
    // Normal transmission
    rfm70_transmitSPI(RFM70_CMD_WR_TX_PLOAD);
  }

  while(len--) {
    rfm70_transmitSPI(*(payload++));
  }

  PORT_SPI |= (1<<CSN);

  return 1;
}


uint8_t rfm70_receivePayload(uint8_t *payload, uint8_t *pipeNumber)
{
  volatile uint8_t len;
  volatile uint8_t status;

  status = rfm70_readRegVal(RFM70_REG_STATUS);

  // Check if new data is in RX FIFO
  if (status & RFM70_STATUS_IRQ_RX_DR) {
    len = rfm70_readRegVal(RFM70_CMD_RX_PL_WID);
    *pipeNumber = (status | RFM70_STATUS_RX_PIPE_NUM) >> 1;
    rfm70_readRegBuf(RFM70_CMD_RD_RX_PLOAD, payload, len);

    // Clear RX FIFO flag in status reg. by writing 1
    rfm70_writeRegVal(RFM70_CMD_WRITE_REG | RFM70_REG_STATUS, RFM70_STATUS_IRQ_RX_DR); 

    return len;
  }

  return 0;
}


uint8_t rfm70_readRegVal(uint8_t cmd)
{
  volatile uint8_t res;
  PORT_SPI &=~ (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  rfm70_transmitSPI(cmd);

  res = rfm70_transmitSPI(0xff);
  PORT_SPI |= (1<<CSN);
  delay_ms(RFM70_CS_DELAY);
  return res;
}


void rfm70_writeRegVal(uint8_t cmd, uint8_t val)
{
  // Put module to PowerDown or StandBy mode
  PORT_CE &= ~(1<<CE);

  PORT_SPI &=~ (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  rfm70_transmitSPI(cmd);
  rfm70_transmitSPI(val);

  PORT_SPI |= (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  // Return to active RX/TX mode
  PORT_CE |= (1<<CE);
}


void rfm70_readRegBuf(uint8_t reg, uint8_t * buf, uint8_t len)
{
  uint8_t status, byte_ctr;

  PORT_SPI &=~ (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  // Select register to write, and read status
  status = rfm70_transmitSPI(reg);

  for(byte_ctr = 0; byte_ctr < len; byte_ctr++) {
    buf[byte_ctr] = rfm70_transmitSPI(0);
  }

  PORT_SPI |= (1<<CSN);
  delay_ms(RFM70_CS_DELAY);
}


void rfm70_writeRegPgmBuf(uint8_t * cmdbuf, uint8_t len) 
{
  // Put module to PowerDown or StandBy mode
  PORT_CE &= ~(1<<CE);

  PORT_SPI &=~ (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  while(len--) {
    rfm70_transmitSPI(pgm_read_byte(cmdbuf++));
  }

  PORT_SPI |= (1<<CSN);
  delay_ms(RFM70_CS_DELAY);

  // Return to active RX/TX mode
  PORT_CE |= (1<<CE);
}


void rfm70_initSPI(uint8_t clk_div)
{
  // set the pin direction to output
  DDR_SPI |= ((1<<MOSI)|(1<<SCK)|(1<<CSN));

  // set the pin direction to input
  DDR_SPI &= ~(1<<MISO);

  // chip select to high
  PORT_SPI |= (1<<CSN);

  // other to low
  PORT_SPI &=~((1<<MOSI)|(1<<SCK)|(1<<MISO));

  // init SPI
  SPCR = ((1<<SPE)|               // SPI Enable
          (0<<SPIE)|              // SPI Interupt Enable
          (0<<DORD)|              // Data Order (0:MSB first / 1:LSB first)
          (1<<MSTR)|              // Master/Slave select   
          (0<<SPR1)|(0<<SPR0)|    // SPI Clock Rate
          (0<<CPOL)|              // Clock Polarity (0:SCK low / 1:SCK hi when idle)
          (0<<CPHA));             // Clock Phase (0:leading / 1:trailing edge sampling)
  
  // det clock divider
  rfm70_spiSetClockDivider(clk_div);
}


void rfm70_spiSetClockDivider(uint8_t rate) 
{
  SPCR = (SPCR & ~SPI_CLOCK_MASK) | (rate & SPI_CLOCK_MASK);
  SPSR = (SPSR & ~SPI_2XCLOCK_MASK) | ((rate >> 2) & SPI_2XCLOCK_MASK);
}


uint8_t rfm70_transmitSPI(uint8_t val)
{
  SPDR = val;
  while((SPSR & (1<<SPIF))==0);

  return SPDR;
}

