/**
 * This library was based on Arduino RFM70 library. 
 *
 * Previously Daniel Weber rewrote it to not use additional 
 * Arduino libraries.
 *
 * I have cleaned it up, rewrote a bit and add more functions
 * that should make it easier for user to run communication
 * over theirs RFM70 modules.
 *
 * Contact to Daniel Weber:
 * Blog: http://projects.web4clans.com
 * Email: daniel.weber@web4clans.com
 *
 * Contact to me:
 * Website: http://www.srcpro.pl
 * e-mail: piotr.krzyzaniak@gmail.com
 *
 * Modified in 2016
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

//************************RFM function parameter constants********************************//
#define WITH_ACK     0x01 // parameter for sendPayload(..): send with ack expectation
#define NO_ACK       0x00 // parameter for sendPayload(..): send without ack expectation
#define MODE_PTX     0x00 // parameter for setMode(mode): set to transmitter
#define MODE_PRX     0x01 // parameter for setMode(mode): set to receiver
#define EN_AA        0x01 // parameter for configRxPipe(..): enable pipe auto ack
#define NO_AA        0x00 // parameter for configRxPipe(..): disable pipe auto ack
#define TX_DPL       0x01 // parameter for configTxPipe(..): enable dynamic payload for PTX
#define TX_SPL       0x00 // parameter for configTxPipe(..): enable static payload for PTX
#define CRC0         0x00 // parameter for configCRC(crc): disable CRC
#define CRC1         0x01 // parameter for configCRC(crc): 1 byte CRC
#define CRC2         0x02 // parameter for configCRC(crc): 2 byte CRC
#define MBPS1        0x01 // parameter for configSpeed(speed): 1Mbps
#define MBPS2        0x02 // parameter for configSpeed(speed): 2Mbps
#define DBMM10       0x00 // parameter for confRfPwr(pwr): -10 dBm
#define DBMM5        0x01 // parameter for confRfPwr(pwr): -5 dBm
#define DBM0         0x02 // parameter for confRfPwr(pwr): 0 dBm
#define DBM5         0x03 // parameter for confRfPwr(pwr): +5 dBm
#define ADR_WIDTH3   0x03 // parameter for confAdrWidth(width): 3 byte
#define ADR_WIDTH4   0x03 // parameter for confAdrWidth(width): 4 byte
#define ADR_WIDTH5   0x03 // parameter for confAdrWidth(width): 5 byte
#define PWR_OFF      0x00 // parameter for setPower(pwr): off
#define PWR_ON       0x01 // parameter for setPower(pwr): on


//************************RFM Definitions************************************************//
#define RFM70_MAX_PACKET_LEN            32    // max value is 32
#define RFM70_BEGIN_INIT_WAIT_MS        300  // pause before Init Registers
#define RFM70_END_INIT_WAIT_MS          100   // pause after init registers
#define RFM70_CS_DELAY                  0     // wait ms after CS pin state change
#define RFM70_ALL_PIPES                 0x3f  // all pipes selected
#define RFM70_PIPE_0                    0x01  // pipe 0 mask
#define RFM70_PIPE_1                    0x02  // pipe 1 mask
#define RFM70_PIPE_2                    0x04  // pipe 2 mask
#define RFM70_PIPE_3                    0x08  // pipe 3 mask
#define RFM70_PIPE_4                    0x10  // pipe 4 mask
#define RFM70_PIPE_5                    0x20  // pipe 5 mask

//************************RFM COMMAND and REGISTER****************************************//
// SPI(RFM70) commands
#define RFM70_CMD_READ_REG              0x00 // Define read command to register
#define RFM70_CMD_WRITE_REG             0x20 // Define write command to register
#define RFM70_CMD_RD_RX_PLOAD           0x61 // Define RX payload command
#define RFM70_CMD_WR_TX_PLOAD           0xA0 // Define TX payload command
#define RFM70_CMD_FLUSH_TX              0xE1 // Define flush TX register command
#define RFM70_CMD_FLUSH_RX              0xE2 // Define flush RX register command
#define RFM70_CMD_REUSE_TX_PL           0xE3 // Define reuse TX payload register command
#define RFM70_CMD_W_TX_PAYLOAD_NOACK    0xB0 // Define TX payload NOACK command
#define RFM70_CMD_W_ACK_PAYLOAD         0xA8 // Define Write ack command
#define RFM70_CMD_ACTIVATE              0x50 // Define feature activation command
#define RFM70_CMD_RX_PL_WID             0x60 // Define received payload width command
#define RFM70_CMD_NOP_NOP               0xFF // Define No Operation, might be used to read status register

// SPI(RFM70) registers(addresses)
#define RFM70_REG_CONFIG                0x00 // 'Config' register address
#define RFM70_REG_EN_AA                 0x01 // 'Enable Auto Acknowledgment' register address
#define RFM70_REG_EN_RXADDR             0x02 // 'Enabled RX addresses' register address
#define RFM70_REG_SETUP_AW              0x03 // 'Setup address width' register address
#define RFM70_REG_SETUP_RETR            0x04 // 'Setup Auto. Retrans' register address
#define RFM70_REG_RF_CH                 0x05 // 'RF channel' register address
#define RFM70_REG_RF_SETUP              0x06 // 'RF setup' register address
#define RFM70_REG_STATUS                0x07 // 'Status' register address
#define RFM70_REG_OBSERVE_TX            0x08 // 'Observe TX' register address
#define RFM70_REG_CD                    0x09 // 'Carrier Detect' register address
#define RFM70_REG_RX_ADDR_P0            0x0A // 'RX address pipe0' register address
#define RFM70_REG_RX_ADDR_P1            0x0B // 'RX address pipe1' register address
#define RFM70_REG_RX_ADDR_P2            0x0C // 'RX address pipe2' register address
#define RFM70_REG_RX_ADDR_P3            0x0D // 'RX address pipe3' register address
#define RFM70_REG_RX_ADDR_P4            0x0E // 'RX address pipe4' register address
#define RFM70_REG_RX_ADDR_P5            0x0F // 'RX address pipe5' register address
#define RFM70_REG_TX_ADDR               0x10 // 'TX address' register address
#define RFM70_REG_RX_PW_P0              0x11 // 'RX payload width, pipe0' register address
#define RFM70_REG_RX_PW_P1              0x12 // 'RX payload width, pipe1' register address
#define RFM70_REG_RX_PW_P2              0x13 // 'RX payload width, pipe2' register address
#define RFM70_REG_RX_PW_P3              0x14 // 'RX payload width, pipe3' register address
#define RFM70_REG_RX_PW_P4              0x15 // 'RX payload width, pipe4' register address
#define RFM70_REG_RX_PW_P5              0x16 // 'RX payload width, pipe5' register address
#define RFM70_REG_FIFO_STATUS           0x17 // 'FIFO Status Register' register address
#define RFM70_REG_DYNPD                 0x1c // 'Enable dynamic payload length' register address
#define RFM70_REG_FEATURE               0x1d // 'Feature' register address

//************************RFM SPI Constants****************************************//
#define RFM70_SPI_CLOCK_DIV4            0x00
#define RFM70_SPI_CLOCK_DIV16           0x01
#define RFM70_SPI_CLOCK_DIV64           0x02
#define RFM70_SPI_CLOCK_DIV128          0x03
#define RFM70_SPI_CLOCK_DIV2            0x04
#define RFM70_SPI_CLOCK_DIV8            0x05
#define RFM70_SPI_CLOCK_DIV32           0x06
#define SPI_CLOCK_MASK                  0x03 // SPR1 = bit 1, SPR0 = bit 0 on SPCR
#define SPI_2XCLOCK_MASK                0x01 // SPI2X = bit 0 on SPSR

#define RFM70_DEFAULT_SPI_CLOCK_DIV     RFM70_SPI_CLOCK_DIV2


// Status Register
#define RFM70_STATUS_IRQ_RX_DR          0x40 // Status bit RX_DR IRQ
#define RFM70_STATUS_IRQ_TX_DS          0x20 // Status bit TX_DS IRQ
#define RFM70_STATUS_IRQ_MAX_RT         0x10 // Status bit MAX_RT IRQ
#define RFM70_STATUS_IRQ_TX_FULL        0x01
#define RFM70_STATUS_RBANK              0x80
#define RFM70_STATUS_RX_PIPE_NUM        0x0E


// Config Register
#define RFM70_CONFIG_MASK_RX_DR         0x40 // Mask interrupt caused by RX_DR when 1
#define RFM70_CONFIG_MASK_TX_DS         0x20 // Mask interrupt caused by TX_DS when 1
#define RFM70_CONFIG_MASK_MAX_RT        0x10 // Mask interrupt caused by MAX_RT when 1
#define RFM70_CONFIG_POWER              0x02 // 1 - Power up ; 0 - Power down
#define RFM70_CONFIG_PRIM_RX            0x01 // 1 - Receiver ; 0 - Transmitter 


// FIFO Status Register
#define RFM70_FIFO_STATUS_TX_REUSE      0x40
#define RFM70_FIFO_STATUS_TX_FULL       0x20
#define RFM70_FIFO_STATUS_TX_EMPTY      0x10
#define RFM70_FIFO_STATUS_RX_FULL       0x02
#define RFM70_FIFO_STATUS_RX_EMPTY      0x01

// Feature Register
#define RFM70_FEATURE_EN_DPL            0x04
#define RFM70_FEATURE_EN_ACK_PAY        0x02
#define RFM70_FEATURE_EN_DYN_ACK        0x01


typedef enum AckTransmissionTypes {
  PALOAD_NORMAL,
  PAYLOAD_WITH_ACK,
  PAYLOAD_WITH_NO_ACK
} AckTypes;


// Configurations for Tx and Rx
void rfm70_defaultTxInit(void);
void rfm70_defaultRxInit(void);

void rfm70_txWithAckInit(void);
void rfm70_rxWithAckInit(void);

void rfm70_txSimpleTransmissionInit(void);
void rfm70_rxSimpleTransmissionInit(uint8_t payloadLen);

// Necessary functions
void rfm70_init(uint8_t clk_div);
void rfm70_initHardware(void);
void rfm70_initRegisters(void);
void rfm70_selectBank(uint8_t bank);
void rfm70_setMode(uint8_t mode);
uint8_t rfm70_getMode(void);
void rfm70_setModeTX(void);
void rfm70_setModeRX(void);
void rfm70_setChannel(uint8_t cnum);
uint8_t rfm70_getChannel(void);
void rfm70_enableAutoAck(uint8_t pipeNumber);
void rfm70_disableAutoAck(uint8_t pipeNumber);
void rfm70_enableDynamicPayload(uint8_t pipeNumber);
void rfm70_disableDynamicPayload(uint8_t pipeNumber);
void rfm70_enablePipe(uint8_t pipeNumber);
void rfm70_disablePipe(uint8_t pipeNumber);
void rfm70_enableTxIrq(void);
void rfm70_disableTxIrq(void);
void rfm70_enableRxIrq(void);
void rfm70_disableRxIrq(void);
void rfm70_enableFeatures(void);
void rfm70_disableFeatures(void);
void rfm70_setPayloadLen(uint8_t pipeNumber, uint8_t payloadLen);
void rfm70_configRfPower(uint8_t pwr);
uint8_t rfm70_getRfPower(void);

void rfm70_flushTxFIFO(void);
void rfm70_flushRxFIFO(void);
uint8_t rfm70_receivePayload(uint8_t *payload, uint8_t *pipeNumber);
uint8_t rfm70_sendPayload(uint8_t *payload, uint8_t len, AckTypes toAck);

uint8_t rfm70_readRegVal(uint8_t cmd);
void rfm70_writeRegVal(uint8_t cmd, uint8_t val);
void rfm70_readRegBuf(uint8_t reg, uint8_t * buf, uint8_t len);
void rfm70_writeRegPgmBuf(uint8_t * cmdbuf, uint8_t len);

void rfm70_initSPI(uint8_t clk_div);
void rfm70_spiSetClockDivider(uint8_t rate);
uint8_t rfm70_transmitSPI(uint8_t val);
